@@ -7,17 +7,22 @@ import urllib.parse
 import importlib.util
 import logging
 import json
-import keycloak
 import environs
 import wsgi_microservice_middleware
+import functools
+import six
+import mimetypes
 import presalytics
 import presalytics.lib.exceptions
 import presalytics.lib.constants as cnst
 import presalytics.client.auth
+import presalytics.client.oidc
 import presalytics.client.presalytics_ooxml_automation.api_client
 import presalytics.client.presalytics_story.api_client
 import presalytics.client.presalytics_doc_converter.api_client
+
 from uuid import uuid4
+from werkzeug.datastructures import FileStorage
 
 logger = logging.getLogger(__name__)
 
@@ -43,11 +48,11 @@ class Client(object):
 
     *A note for server-side development*:
 
-    By default, the client class automatically caches tokens in a file called 
+    The client class can automatically cache tokens in a file called 
     "token.json", located in the python's current working directory.  This is done so
-    users running scripts do not have to acquire a new token every time an API call is
-    made. If building a client to operate in a multi-user environment, this behavior 
-    should be turned off so that user do not pull one another's tokens.
+    users running scripts accross multiple client instances do not have to acquire a new token 
+    every time an API call is made. If building a client to operate in a multi-user environment, 
+    this behavior should be turned off so that one user cannot not pull one another's tokens.
     To do this, ensure the following parameters are pass to the configuration either 
     via initialization or in a `presalytics.CONFIG` file: 
     
@@ -92,7 +97,7 @@ class Client(object):
         raise an `presalytics.lib.exceptions.InvalidTokenException`.
 
     cache_tokens : bool, optional
-        Defaults to True.  Toggles whether or the client should cache its acquired tokens in file called "token.json"
+        Defaults to False.  Toggles whether or the client should cache its acquired tokens in file called "token.json"
         in the current working directory.  Minimizes the number of times a user is required to login.  Set to False
         in multi-user environments.
 
@@ -180,9 +185,8 @@ class Client(object):
         Indicates whether a this client can obtain tokens from auth.presalytics.io without a user under
         OpenID Connect grant type "confidential_client".  Requires a `client_secret`.  Default is False. 
 
-    oidc : `keycloak.KeycloakOpenID`
-        A middleware class for to help acquire tokens from auth.presalytics.io.  For more information, review
-        the docs at https://python-keycloak.readthedocs.io/en/latest/ .
+    oidc : `presalytics.client.oidc.OidcClient`
+        A middleware class to help acquire and validate tokens from login.presalytics.io.
 
     token_util : `presalytics.client.auth.TokenUtil`
         A handler for managing an caching tokens acquired from auth.presalytics.io.
@@ -212,7 +216,9 @@ class Client(object):
             password=None,
             delegate_login=False,
             token=None,
-            cache_tokens=True,
+            cache_tokens=False,
+            client_id=None,
+            client_secret=None,
             **kwargs):
     
         if username:
@@ -224,7 +230,8 @@ class Client(object):
                 if token:
                     self.username = None
                 else:
-                    raise presalytics.lib.exceptions.MissingConfigException("Configuration variable PRESALYTICS_USERNAME is required when client is not initalized with an API token.  Please reconfigure and retry.")
+                    if not client_secret:
+                        raise presalytics.lib.exceptions.MissingConfigException("when not passing tokens directly, a clien must have either a client_secrect or a username")
         try:
             if password:
                 self.password = password
@@ -235,22 +242,28 @@ class Client(object):
             self.password = None
             self.direct_grant = False
         try:
-            self.client_id = presalytics.CONFIG['CLIENT_ID']
+            if client_id:
+                self.client_id = client_id
+            else:
+                self.client_id = presalytics.CONFIG['CLIENT_ID']
         except KeyError:
             self.client_id = cnst.DEFAULT_CLIENT_ID
         try:
-            self.client_secret = presalytics.CONFIG['CLIENT_SECRET']
+            if client_secret:
+                self.client_secret = client_secret
+            else:
+                self.client_secret = presalytics.CONFIG['CLIENT_SECRET']
             self.confidential_client = True
         except KeyError:
             self.client_secret = None
             self.confidential_client = False
-        self.verify_https = presalytics.CONFIG.get("VERIFY_HTTPS", True)
+
         try:
             self.site_host = presalytics.CONFIG["HOSTS"]["SITE"]
         except KeyError:
             self.site_host = cnst.SITE_HOST
-        self.login_sleep_interval = 5  # seconds
-        self.login_timeout = 60  # seconds
+
+
         try:
             self.redirect_uri = presalytics.CONFIG["REDIRECT_URI"]
         except KeyError:
@@ -259,20 +272,25 @@ class Client(object):
             self._delegate_login = True
         else:
             self._delegate_login = False
-        self.oidc = keycloak.KeycloakOpenID(
-            server_url=cnst.OIDC_AUTH_HOST,
-            realm_name=cnst.OIDC_REALM,
+        self.oidc = presalytics.client.oidc.OidcClient(
             client_id=self.client_id,
-            verify=self.verify_https
+            client_secret=self.client_secret
         )
         if presalytics.CONFIG.get("CACHE_TOKENS", None):
             cache_tokens = presalytics.CONFIG.get("CACHE_TOKENS")
         self.token_util = presalytics.client.auth.TokenUtil(token_cache=cache_tokens)
         if token:
-            if token.get('access_token_expire_time', None):
+            #  Assume if token is passed as string, then it's an access token
+            if isinstance(token, str):
+                self.token_util.token = {"access_token": token}
+            
+            # if token is a dictionary with an 'access_token_expire_time' key, it's previous been processed / deserialized
+            elif token.get('access_token_expire_time', None):
                 self.token_util.token = token
+
+            # if token has an 'expires_in' key, if has not been deserialized
             elif token.get('expires_in', None):
-                self.token_util.process_keycloak_token(token)
+                self.token_util.process_token(token)
             else:
                 raise presalytics.lib.exceptions.InvalidTokenException(message="Unknown token format.")
             if self.token_util.token_cache:
@@ -291,107 +309,13 @@ class Client(object):
         """
         Triggers a an attempt to acquire an API token based on the the client configuration
         """
-        try:
-            if self.direct_grant:
-                keycloak_token = self.oidc.token(username=self.username, password=self.password)
-            else:
-                keycloak_token = self._get_new_token_browser()
-        except keycloak.exceptions.KeycloakGetError as e:
-            if e.response_code == 404:
-                logger.error("Received reponse code 404 from api authentication.  Indicates username does not exist.  Please recheck config and try again.")
-            elif e.response_code == 401:
-                logger.error("Received 401 response code from api authentication.  This user's access to this resource is unauthorized")
-            elif e.response_code >= 500:
-                logger.error("Server error.  Please try again later.")
-            else:
-                logger.error(e.error_message)
-            raise presalytics.lib.exceptions.MissingConfigException(e.error_message)
-        self.token_util.process_keycloak_token(keycloak_token)
+        if self.direct_grant:
+            token = self.oidc.token(username=self.username, password=self.password)
+        else:
+            token = self.oidc.token(username=self.username)
+        self.token_util.process_token(token)
         return self.token_util.token
 
-    def exchange_token(self, original_token, audience=None):
-        """
-        Conduct IETF [token exchange](https://tools.ietf.org/id/draft-ietf-oauth-token-exchange-19.html).
-        Use for client applications to modify the scope of token granted by user authenication so they 
-        do not get misused by 3rd party services. 
-
-        Parameters
-        ----------
-
-        original_token : str
-            A str with JWT Bearer access token obtained from auth.presalytics.io
-
-        audience : str
-            A str naming the referrring to audience of the server that will consume the new token
-
-        Returns:
-        ---------
-        A str containing a JWT Bearer token with reduced scope
-
-        """
-        self.oidc.decode_token(original_token, cnst.JWT_KEY)
-        kwargs = {
-            "client_id": self.client_id,
-            "client_secret": self.client_secret,
-            "grant_type": "urn:ietf:params:oauth:grant-type:token-exchange",
-            "subject_token": original_token
-        }
-        if audience is not None:
-            kwargs.update(
-                {
-                    "audience": audience
-                }
-            )
-        keycloak_token = self.oidc.token(**kwargs)
-        self.token_util.process_keycloak_token(keycloak_token)
-        return self.token_util.token["access_token"]
-
-    def _get_new_token_browser(self):
-        """
-        Opens browser in on presalytics.io and prompts for user login.
-        Retrieves authorization code from website and obtains api token
-        """
-        api_otp = uuid4()
-        query = {
-            "api_otp": api_otp,
-            "client_id": self.client_id,
-            "next": urllib.parse.urlparse(self.redirect_uri).path
-        }
-        query_string = '?{}'.format(urllib.parse.urlencode(query))
-        url = urllib.parse.urljoin(self.site_host, urllib.parse.urljoin(cnst.LOGIN_PATH, query_string))
-        logger.info("Opening new browser tab.  Please input login credentials...")
-        logger.info("Requesting login at location: <{0}>".format(url))
-        webbrowser.open_new_tab(url)
-        auth_code = None
-        payload = {
-            "username": self.username,
-            "api_otp": str(api_otp),
-            "client_id": self.client_id
-        }
-
-        auth_code_url = urllib.parse.urljoin(self.site_host, cnst.API_CODE_URL)
-        interval = 0
-        while True:
-            response = requests.post(auth_code_url, json=payload, verify=self.verify_https)
-            if response.status_code != 200:
-                if interval <= self.login_timeout:
-                    interval += self.login_sleep_interval
-                    time.sleep(self.login_sleep_interval)
-                else:
-                    raise presalytics.lib.exceptions.LoginTimeout()
-            else:
-                data = json.loads(response.content)
-                break
-        if data.get("token", None):
-            #  pick up otp token off presalytics backend
-            token = data.get("token") 
-        else:
-            # support oidc authorization code flow
-            auth_code = data["authorization_code"]
-            token = self.oidc.token(username=self.username, grant_type="authorization_code", code=auth_code, redirect_uri=self.redirect_uri)
-        if token:
-            logger.info("Login Succeeded.  Proceeding.")
-        return token
 
     def refresh_token(self):
         """
@@ -400,20 +324,21 @@ class Client(object):
         an `presalytics.lib.exceptions.InvalidTokenException` when `deletegate_login` is True.
         """
         if self.token_util.is_api_access_token_expired():
-            if self.token_util.is_api_refresh_token_expired():
-                if not self._delegate_login:  # do not automatically log in at init
-                    self.login()
-                else:
-                    raise presalytics.lib.exceptions.InvalidTokenException(message="This token has expired.")
+            if self.token_util.token.get('refresh_token', None):
+                refresh_token = self.token_util.token["refresh_token"]
+                token = self.oidc.refresh_token(refresh_token)
+                self.token_util.process_token(token)
+                logger.debug("Refresh token granted successfully.")
             else:
-                try:
-                    refresh_token = self.token_util.token["refresh_token"]
-                    keycloak_token = self.oidc.refresh_token(refresh_token)
-                    self.token_util.process_keycloak_token(keycloak_token)
-                    logger.debug("Refresh token granted successfully.")
-                except keycloak.exceptions.KeycloakGetError:
-                    if not self._delegate_login:
-                        self.login()
+                if self.direct_grant:
+                    token = self.oidc.token(username=self.username, password=self.password)
+                elif self.confidential_client:
+                    token = self.oidc.client_credentials_token()
+                elif self._delegate_login:
+                    raise presalytics.lib.exceptions.ApiError("Unauthorized. Token has expired", status_code=401)
+                else:
+                    token = self.oidc.token(username=self.username)
+                self.token_util.process_token(token)
             if self.token_util.token_cache:
                 self.token_util._put_token_file()
         return self.token_util.token
@@ -482,6 +407,26 @@ class Client(object):
         with open(filepath, 'wb') as f:
             f.write(response.data)
 
+    def get_client_info(self):
+        """
+        Convenience method returning information about this client to pass to downstream objects, e.g.,
+        components and new client instances
+
+        Returns
+        ----------
+        A dictionary containing instances values:
+            - token: self.token_util.token
+            - client_id: self.client_id
+            - cache_tokens: self.token_util.token_cache
+            - delegate_login: self.delegate login
+        """
+        return {
+            "token": self.token_util.token,
+            "client_id": self.client_id,
+            "cache_tokens": self.token_util.token_cache,
+            "delegate_login": self._delegate_login
+        }
+
 class DocConverterApiClientWithAuth(presalytics.client.auth.AuthenticationMixIn, presalytics.client.presalytics_doc_converter.api_client.ApiClient):
     """
     Wraps `presalytics.client.presalytics_doc_converter.api_client.ApiClient` with 
@@ -503,6 +448,40 @@ class OoxmlAutomationApiClientWithAuth(presalytics.client.auth.AuthenticationMix
         presalytics.client.presalytics_ooxml_automation.api_client.ApiClient.__init__(self)
         self.update_configuration()
 
+    def files_parameters(self, files=None):
+        """Builds form parameters.
+
+        This override method expands the capabilites of codegen filehandler to
+        accept a `werkzeug.datastructures.FileStorage` object.
+
+        :param files: File parameters. Either a string file path or a `werkzeug.datastructures.FileStorage` object
+        :return: Form parameters with files.
+        """
+        params = []
+
+        if files:
+            for k, v in six.iteritems(files):
+                if not v:
+                    continue
+                if type(v) is str or type(v) is list:
+                    file_names = v if type(v) is list else [v]
+                    for n in file_names:
+                        with open(n, 'rb') as f:
+                            filename = os.path.basename(f.name)
+                            filedata = f.read()
+
+                else:
+                    if type(v) is FileStorage:
+                        filename = v.filename
+                        v.stream.seek(0)
+                        filedata = v.stream.read()
+                    else:
+                        raise AttributeError("Invalid File Object")
+                mimetype = (mimetypes.guess_type(filename)[0] or 'application/octet-stream')
+                params.append(tuple([k, tuple([filename, filedata, mimetype])]))
+
+        return params
+
 class StoryApiClientWithAuth(presalytics.client.auth.AuthenticationMixIn, presalytics.client.presalytics_story.api_client.ApiClient):
     """
     Wraps `presalytics.client.presalytics_story.api_client.ApiClient` with
@@ -514,6 +493,15 @@ class StoryApiClientWithAuth(presalytics.client.auth.AuthenticationMixIn, presal
         self.update_configuration()
 
 
+@functools.lru_cache(maxsize=None)
+def get_client():
+    """
+    Caches a client instance for default parameters set in `presalytics.CONFIG`.
+
+    DO NOT use in server-side operation
+    """
+    client = presalytics.Client()
+    return client
 
 
         
