@@ -20,7 +20,7 @@ logger = logging.getLogger(__name__)
 
 
 class TokenUtil(object):
-    def __init__(self, token=None, token_file=None, token_cache=True):
+    def __init__(self, token=None, token_file=None, token_cache=False):
         self.token_cache = token_cache
         if token_file is None:
             self.token_file = presalytics.lib.constants.TOKEN_FILE
@@ -32,41 +32,43 @@ class TokenUtil(object):
             pass
         if token is not None:
             try:
-                self.token = {
-                    'access_token': token['access_token'],
-                    'access_token_expire_time': token["access_token_expire_time"],
-                    'refresh_token': token["refresh_token"],
-                    'refresh_token_expire_time': token["refresh_token_expire_time"]
-                }
+                self.process_token(token)    
                 if self.token_cache:
                     self._put_token_file()
             except Exception:
                 raise presalytics.lib.exceptions.MisConfiguredTokenException()
+        else:
+            self.token = {}
 
     def is_api_access_token_expired(self):
         try:
-            expire_datetime = dateutil.parser.parse(self.token['access_token_expire_time']).astimezone(datetime.timezone.utc)
+            if not self.token.get('access_token_expire_time', None):
+                if self.token.get('access_token', None):
+                    return False  # If expire time unknown, instruct client to call endpoint
+                else:
+                    return True  # If no token, instruct client to acquire token
+            token_expiry = self.token['access_token_expire_time']
+            if isinstance(token_expiry, datetime.datetime):
+                expire_datetime = token_expiry.astimezone(datetime.timezone.utc)
+            elif isinstance(token_expiry, str):
+                expire_datetime = dateutil.parser.parse(self.token['access_token_expire_time']).astimezone(datetime.timezone.utc)
+            else:
+                logger.error("Token expire time not of type str (isoformat) or datetime.datetime")
+                return True  # Get a new token on type error
             if expire_datetime < datetime.datetime.utcnow().astimezone(datetime.timezone.utc):
                 return True
-            return False
-        except Exception:
-            return True
-
-    def is_api_refresh_token_expired(self):
-        try:
-            expire_datetime = dateutil.parser.parse(self.token['refresh_token_expire_time']).astimezone(datetime.timezone.utc)
-            if expire_datetime < datetime.datetime.utcnow().astimezone(datetime.timezone.utc):
-                return True
-            return False
-        except Exception:
-            return True
+            else:
+                return False
+        except Exception as ex:
+            logger.exception(ex)
+            return True # Get a new token on unknown errors
 
     def _load_token_file(self):
         try:
             if self.token_cache:
                 self.token = self.load_token_from_file(self.token_file)
         except Exception:
-            logger.error("Unable to load token from cache.  If you do want intend to cache tokens, use configuration CACHE_TOKENS=False")
+            logger.error("Unable to load token from cache.  If you do intend to cache tokens, use configuration CACHE_TOKENS=False")
 
     def _put_token_file(self):
         try:
@@ -75,17 +77,25 @@ class TokenUtil(object):
         except Exception:
             logger.error("Failed to cache token.  Likely a write permissions error for the filesystem.")
 
-    def process_keycloak_token(self, keycloak_token):
-        access_token_expire_time = datetime.datetime.utcnow().astimezone(datetime.timezone.utc) + datetime.timedelta(seconds=keycloak_token['expires_in'])
-        refresh_token_expire_time = datetime.datetime.utcnow().astimezone(datetime.timezone.utc) + datetime.timedelta(seconds=keycloak_token['refresh_expires_in'])
-
+    def process_token(self, token):
         self.token = {
-            'access_token': keycloak_token['access_token'],
-            'refresh_token': keycloak_token['refresh_token'],
-            'access_token_expire_time': access_token_expire_time.isoformat(),
-            'refresh_token_expire_time': refresh_token_expire_time.isoformat()
+            'access_token': token['access_token']
         }
-
+        if not token.get('access_token_expire_time', None):
+            if token.get('expires_in', None):
+                access_token_expire_time = datetime.datetime.utcnow().astimezone(datetime.timezone.utc) + datetime.timedelta(seconds=token['expires_in'])
+            else:
+                pass    # TODO: add logic to introspect token for expire time 
+        else:
+            access_token_expire_time = token["access_token_expire_time"]
+        if access_token_expire_time:
+            self.token.update({
+                "access_token_expire_time": access_token_expire_time
+            })
+        if token.get('refresh_token', None):
+            self.token.update({
+                "refresh_token": token["refresh_token"]
+            })
         return self.token
 
     @staticmethod
@@ -123,7 +133,7 @@ class AuthenticationMixIn(object):
             Missing reference to Client class.  Client was like garbage collected by the intepreter.\n  
             Please initialize the Client class on its own line to avoid this error.  For example:\n\n
             client = presalytics.Client()\n
-            story = client.story.story_id_get(story_id)\n\n
+            story = client.story.story_id_get(story_id)
             """
             raise presalytics.lib.exceptions.InvalidConfigurationError(message=message)
         auth_header = self.parent().get_auth_header()
@@ -139,12 +149,25 @@ class AuthenticationMixIn(object):
 
             endpoint = self.configuration.host + resource_path
             logger.info("Sending {0} message to {1}. Request Id: {2}".format(method, endpoint, request_id))
-            response = super(AuthenticationMixIn, self).call_api(
-                resource_path, method, path_params,
-                query_params, header_params, body, post_params, files, response_type,
-                auth_settings, async_req, _return_http_data_only, collection_formats, _preload_content,
-                _request_timeout, _host
+            call_args = (
+                resource_path, 
+                method, 
+                path_params,
+                query_params, 
+                header_params, 
+                body, 
+                post_params, 
+                files, 
+                response_type,
+                auth_settings, 
+                async_req, 
+                _return_http_data_only, 
+                collection_formats, 
+                _preload_content,
+                _request_timeout, 
+                _host
             )
+            response = super(AuthenticationMixIn, self).call_api(*call_args)
             logger.info("{0} response received from {1}".format(method, endpoint))
             return response
         except Exception as e:
@@ -158,6 +181,10 @@ class AuthenticationMixIn(object):
                         d = e.body + addendum
                     e.body = json.dumps(d)
                     logging.error(d)
+                    if d["status"] == 401:
+                        logging.debug("Refeshing token from unauathorized call and retrying")
+                        self.parent().refresh_token()
+                        return super(AuthenticationMixIn, self).call_api(*call_args)
                 except Exception:
                     pass
                 if self._ignore_api_exceptions:
@@ -168,10 +195,6 @@ class AuthenticationMixIn(object):
                 t, v, tb = sys.exc_info()
                 six.reraise(t, v, tb)
 
-    # def update_timezones(self, response)
-
-    #     return response
-
     def update_configuration(self):
         """
         updates the configuration from the base api client to parameters contained in the config file.
