@@ -7,6 +7,8 @@ env.read_env()
 PRESALYTICS = {
     'USERNAME': os.environ['PRESALYTICS_USERNAME'],
     'PASSWORD': os.environ['PRESALYTICS_PASSWORD'],
+    'CLIENT_ID': os.environ.get('CLIENT_ID'),
+    'CLIENT_SECRET': os.environ.get('CLIENT_SECRET'),
     'HOSTS': {}
 }
 
